/*
Copyright (C) 1995 M. A. S. Elmohamed,
Northeast Parallel Architectures Center, Syracuse University.
saleh@npac.syr.edu
*/


#include <stdio.h>
#include <malloc.h>
#ifndef MSDOS
#include <strings.h>
#endif
#include "misc.h"
#include "config.h"
#include "applvar.h"

applvar_t applvar_create(char *directory, char *config_file)
{
  applvar_t av = (applvar_t)0;
  char buf[SCRATCH_BUF_LEN];

  av = (applvar_t) xcalloc(1,sizeof(applvar_s));

  /* Get the name of the students file and open and input stream on the file */
  if (applvar_find_config_entry(config_file, KEY_STUDENTS, buf) == 0) {
    av->students_file = build_path(directory, buf);
    av->students_fp = xfopen(av->students_file,"r");
  }
  else {
    fprintf(stderr,"%s: configuration parameter '%s' missing\n",
	    program,KEY_STUDENTS);
    exit(1);
  }
    
  /*----------------------------*/
  /* ... and the advisors files */
  /*----------------------------*/
  if (applvar_find_config_entry(config_file, KEY_PROFESSORS, buf) == 0) {
    av->professors_file = build_path(directory, buf);
    av->professors_fp = xfopen(av->professors_file,"r");
  }
  else {
    fprintf(stderr,"%s: configuration parameter '%s' missing\n",
	    program,KEY_PROFESSORS);
    exit(1);
  }
    
  /*--------------------------------------------*/
  /* ... and the advisory committee groups file */
  /*--------------------------------------------*/
  if (applvar_find_config_entry(config_file, KEY_THESIS_GROUPS, buf) == 0) {
    av->thesis_groups_file = build_path(directory, buf);
    av->thesis_groups_fp = xfopen(av->thesis_groups_file,"r");
  }
  else {
    fprintf(stderr,"%s: configuration parameter '%s' missing\n",
	    program,KEY_THESIS_GROUPS);
    exit(1);
  }
    
  /* ... and the actual meeting times required file */
  if (applvar_find_config_entry(config_file, KEY_TIME_SLOTS, buf) == 0) {
    av->time_slots_file = build_path(directory, buf);
    av->time_slots_fp = xfopen(av->time_slots_file,"r");
  }
  else {
    fprintf(stderr,"%s: configuration parameter '%s' missing\n",
	    program,KEY_TIME_SLOTS);
    exit(1);
  }    

  /*------------------------*/
  /* Open the output file stream (If none specified, stdout is used) */
  /*------------------------*/
  if (applvar_find_config_entry(config_file, KEY_OUTPUT_FILE, buf) == 0) {
    av->output_file = build_path(directory, buf);
    av->output_fp = xfopen(av->output_file,"w");
  }
  else {
    av->output_file = "[stdout]";
    av->output_fp = stdout;
  }    

  /*----------------------------------------------
     max_iters is the number of successes to process, (e.g. if the
     first solution found is not ideal for your purposes, you would
     set max_iters to 2 and then evaluate the second solution).
    ---------------------------------------------*/
  if (applvar_find_config_entry(config_file, KEY_MAX_ITERS, buf) == 0) {
    av->max_iters = atoi(buf);
  }
  else {
    fprintf(stderr,"%s: configuration parameter '%s' missing\n",
	    program,KEY_MAX_ITERS);
    exit(1);
  }    

  /*------------------------*/
  /* Get the number of rooms we are constrained to use. */
  /*------------------------*/
  if (applvar_find_config_entry(config_file, KEY_NUMBER_OF_ROOMS, buf) == 0) {
    av->number_of_rooms = atoi(buf);
  }
  else {
    fprintf(stderr,"%s: configuration parameter '%s' missing\n",
	    program,KEY_NUMBER_OF_ROOMS);
    exit(1);
  }    

  /*-------------------------
     Get the minimum idle time (hours) after a meeting before a room
     can be re-used.  This parameter allows students to set up their
     presentations when the room is idle.
    --------------------------*/
  if (applvar_find_config_entry(config_file, KEY_ROOM_IDLE_TIME, buf) == 0) {
    av->room_idle_time = atoi(buf);
  }
  else {
    fprintf(stderr,"%s: configuration parameter '%s' missing\n",
	    program,KEY_ROOM_IDLE_TIME);
    exit(1);
  }    

  /*--------------------------
     A usefull debugging tool, print_all_results prints copious amounts
     of intermediate results, as well as each successful result (when 
     max_iters is greater than one).
    --------------------------*/
  if (applvar_find_config_entry(config_file, KEY_PRINT_ALL_RESULTS, buf) == 0)
    av->print_all_results = 1;
  else
    av->print_all_results = 0;

  return(av);
}

/* ********************************************************************** */

int applvar_find_config_entry(char *config_file, char *key, char *val)
{
  FILE *cfp;
  char **field = (char **)0;
  char buf[SCRATCH_BUF_LEN], *s;
  int nfields, i, found = -1;

  cfp = xfopen(config_file,"r");
  while (getline(buf,sizeof(buf),cfp) != (char *)0) {
    rm_lead_ws(buf);			/* remove leading whitespace */
    if (*buf == '#' || *buf == '\0')	/* comment and empty lines ignored */
      continue;
    field = split_ws_separated_line(field, buf, &nfields);
    if (strcasecmp(field[0],key) == 0) {
      /* concatenate all fields */
      for (i = 1; i < nfields; i++) {
	for (s = field[i]; *s; )
	  *val++ = *s++;
	*val++ = (i+1 == nfields ? '\0' : ' ');
      }
      found++;
      break;
    }
  }
  xfclose(cfp);
  /* release allocated space */
  free_fields(field, nfields);
  return(found);
}
