/*
Copyright (C) 1995 M. A. S. Elmohamed,
Northeast Parallel Architectures Center, Syracuse University.
saleh@npac.syr.edu
*/


#ifndef DEFINED_APPLVAR_H
#define DEFINED_APPLVAR_H

/* **********************************************************************/
/* ANSI -- because we were too lazy to lint */
#ifndef _PARAMS
#if defined(ANSI) || defined(_ANSI_) || defined(__STDC__)
#define _PARAMS(args) args
#else
#define _PARAMS(args) ()
#endif
#endif /* _PARAMS */

#ifndef _K_AND_R_C_
#if !(defined(ANSI) || defined(_ANSI_) || defined(__STDC__))
#define _K_AND_R_C_
#define void
#endif
#endif /* _K_AND_R_C_ */
/* **********************************************************************/

/*
The applvar data structure holds application variables.
An application variable is a variable that is needed throughout
the application.  In an orderly world, the variable 'program'
would probably be in this structure, but the weight of tradition
weighs heavily on my shoulders...
*/

typedef struct applvar_struc {
  /* file names */
  char *students_file;		        /* students names and schedules */
  char *professors_file;		/* professors names and schedules */
  char *thesis_groups_file;	        /* students and advisory groups */
  char *time_slots_file;		/* time slots to be filled */
  char *output_file;			/* output file name */
  /* file stream pointers */
  FILE *students_fp;		        /* students names and schedules */
  FILE *professors_fp;			/* professors names and schedules */
  FILE *thesis_groups_fp;	        /* students and advisory groups */
  FILE *time_slots_fp;			/* time slots to be filled */  
  FILE *output_fp;			/* output stream */
  int max_iters;			/* maximum number of iterations */
  int print_all_results;
  int number_of_rooms;
  int room_idle_time;
} applvar_s, *applvar_t;

#define KEY_PRINT_ALL_RESULTS	"print_all_results"
#define KEY_STUDENTS		"students_file"
#define KEY_PROFESSORS		"advisors_file"
#define KEY_THESIS_GROUPS	"thesis_groups_file"
#define KEY_TIME_SLOTS		"time_slots_file"
#define KEY_OUTPUT_FILE		"output_file"
#define KEY_NUMBER_OF_ROOMS	"number_of_rooms"
#define KEY_ROOM_IDLE_TIME	"room_idle_time"
#define KEY_MAX_ITERS		"max_iters"

applvar_t applvar_create _PARAMS((char *directory, char *config_file));
int applvar_find_config_entry _PARAMS((char *config_file, 
				       char *key, char *val));

#endif	/* DEFINED_APPLVAR_H */
