/*
Copyright (C) 1995 M. A. S. Elmohamed,
Northeast Parallel Architectures Center, Syracuse University.
saleh@npac.syr.edu
*/

#include <stdio.h>
#include <ctype.h>
#include "misc.h"

#ifdef MSDOS
int strcasecmp(s1, s2)
     char *s1;
     char *s2;
{
  register int i;
  int len1,len2;
  char c1,c2;

  for (i=0, len1=strlen(s1), len2=strlen(s2); i < len1 && i < len2; i++) {
    c1 = *(s1 + i);
    c2 = *(s2 + i);
    if (isupper(c1))
      c1 = tolower(c1);
    if (isupper(c2))
      c2 = tolower(c2);
    if (c1 > c2)
      return(1);
    else if (c1 < c2)
      return(-1);
  }
  if (len1 > len2)
    return(1);
  else if (len1 < len2)
    return(-1);
  return(0);
}
#endif /* MSDOS */

char *getline(buf,len,fp)
     char *buf;
     int len;
     FILE *fp;
{
  if (fgets(buf,len,fp) == NULL)
    return(NULL);
  len = strlen(buf);
  if (len && buf[len - 1] == '\n') {
    buf[--len] = '\0';
  }
  
  /* special case for CRLF encoded files */
  if (len && buf[len - 1] == '\r') {
    buf[--len] = '\0';
  }
  return(buf);
}

char *rm_ws(char *buf)
{
  register char *p = buf, *s;

  if (p != (char *)0) {
    for (s = p; *p; p++)
      if (*p != ' ' && *p != '\t')
	*s++ = *p;
    *s = *p;
  }
  return(buf);
}

char *rm_lead_ws(char *buf)
{
  register char *p = buf, *s;
  register int within = 0;

  if (p != (char *)0) {
    for (s = p; *p; p++)
      if (within || (*p != ' ' && *p != '\t')) {
	*s++ = *p;
	within++;
      }
    *s = *p;
  }
  return(buf);
}

char *rm_trail_ws(char *buf)
{
  register int i;

  if (buf != (char *)0)
    for (i = strlen(buf) - 1; i >= 0; i--)
      if (buf[i] != ' ' && buf[i] != '\t') {
	buf[i+1] = '\0';
	break;
      }
  return(buf);
}

char *rm_lead_trail_ws(char *buf)
{
  rm_lead_ws(buf);
  return(rm_trail_ws(buf));
}

char *lowercase(char *buf)
{
  register char *p = buf;

  if (p != (char *)0)
    for (; *p; p++)
      if (isupper(*p))
	*p = tolower(*p);
  return(buf);
}

/*
   splits a whitespace separated line into fields.
   BUGS: should be a little more general. Maybe later.
*/
char **split_ws_separated_line(char **field, char *line, int *nfields)
{
  int i,point,len,nf,nslots;
  char *string;

  nf =*nfields;
  len = strlen(line);
  nslots = len / 2 + 1;
  if (field == NULL)
    field = (char **)xcalloc(nslots,sizeof(char *));
  else {
    field = (char **)xrealloc((char *)field,nslots * sizeof(char *));
    for (i = nf; i < nslots; i++)
      field[i] = (char *) 0;
  }

  string = xmalloc(len + 1);
  for (nf = 0,point = 0; nf < nslots; nf++) {
    while (line[point] == ' ' || line[point] == '\t') point++;
    string_parse_ws_separated_token(line, &point, string);
    if (string[0] == '\0')
      break;
    
    len = strlen(string);
    if (field[nf] != NULL)
      xfree(field[nf]);
    field[nf] = xmalloc(len + 1);
    strcpy(field[nf],string);
  }
  for (i = nf; i < nslots; i++) {
    if (field[i] != NULL)
      free(field[i]);
  }
  field = (char **)xrealloc((char *)field,nf * sizeof(char *));

  free(string);

  *nfields = nf;
  return(field);
}

char **split_line(char **field, char *line, int *nfields, char sep)
{
  int i,point,len,nf,nslots;
  char *string;

  nf =*nfields;
  len = strlen(line);
  nslots = len / 2 + 1;
  if (field == NULL)
    field = (char **)xcalloc(nslots,sizeof(char *));
  else {
    field = (char **)xrealloc((char *)field,nslots * sizeof(char *));
    for (i = nf; i < nslots; i++)
      field[i] = (char *) 0;
  }

  string = xmalloc(len + 1);
  for (nf = 0,point = 0; nf < nslots; nf++) {
    string_parsetoken(line, &point, string, sep);
    if (string[0] == '\0')
      break;
    
    len = strlen(string);
    if (field[nf] != NULL)
      free(field[nf]);
    field[nf] = xmalloc(len + 1);
    strcpy(field[nf],string);
  }
  for (i = nf; i < nslots; i++) {
    if (field[i] != NULL)
      free(field[i]);
  }
  field = (char **)xrealloc((char *)field,nf * sizeof(char *));

  free(string);

  *nfields = nf;
  return(field);
}

void free_fields(char **field, int nfields)
{
  if (field != NULL) {
    for (; nfields > 0; nfields--)
      free(field[nfields-1]);
    free(field);
  }
}

char *string_parse_ws_separated_token(char *buf, int *point, char  *string)
{
  char *start;
  char *tail, *head;
  
  tail = head = string;
  start = buf;			/* remember start of buffer */
  buf = buf + *point;		/* position start of buffer */
  
  /* skip leading white space */
  while (*buf == ' ' || *buf == '\t')
    buf++;
  
  for (; *buf && *buf != ' ' && *buf != '\t'; buf++)
    *(tail++) = *buf;
  if (*buf && (*buf == ' ' || *buf == '\t'))
    buf++;					/* move past separator */
  *tail = '\0';
  
  /* remove trailing white space */
  while (--tail >= head && (*tail == ' ' || *tail == '\t'))
    *tail = '\0';
  
  *point = (int) (buf - start);
  return(head);
}

char *string_parsetoken(buf, point, string, sep)
     char *buf;
     char *string;
     int *point;
     char sep;
{
  char *start;
  char *tail, *head;
  
  tail = head = string;
  start = buf;			/* remember start of buffer */
  buf = buf + *point;		/* position start of buffer */
  
  /* skip leading white space */
  if (sep == ' ' || sep == '\t') {
    while (*buf == ' ')
      buf++;
  }
  
  for (; *buf && *buf != sep; buf++)
    *(tail++) = *buf;
  if (*buf && *buf == sep)
    buf++;					/* move past separator */
  *tail = '\0';
  
  /* remove trailing white space */
  if (sep == ' ' || sep == '\t') {
    while (--tail >= head && *tail == ' ')
      *tail = '\0';
  }
  
  *point = (int) (buf - start);
  return(head);
}

char *xcalloc(n,c)
     unsigned n;
     unsigned c;
{
  extern char *calloc();
  char *p;
  
  if ((p = calloc(n,c)) == NULL)
    perror("out of memory in xcalloc");
  
  return(p);
}

char *xmalloc(n)
     unsigned n;
{
  extern char *malloc();
  char *p;
  
  if ((p = malloc(n)) == NULL)
    perror("out of memory in xmalloc");
  
  return(p);
}

char *xrealloc(p, newsize)
     char *p;
     unsigned newsize;
{
  extern char *realloc();
  char *r;
  
  if ((r = realloc(p, newsize)) == NULL)
    perror("out of memory in xrealloc");
  return(r);
}	

void xfree(s)
     char *s;
{
  if (s != NULL)
    free(s);
}

char *basename(path,suffix,string)
     char *path;    
     char *suffix;
     char *string;
{
  char *malloc();
  int l,i,j;

  if (path == NULL)
    return(NULL);

  l = strlen(path);
  if (l == 0 || path[l - 1] == '\\' || path[l - 1] == '/')
    return(NULL);	/* path is not a file--it may be a directory */

  
  if (suffix != NULL) {
    for (i = l - 1, j = strlen(suffix) - 1; i >= 0 && j >= 0; i--, j--)
      if (path[i] != suffix[j])
	break;
    if (j < 0) {
      l = i + 1;
      path[l] = '\0';
    }
  }
    
  for (i = l - 1; i >= 0; i--)
    if (path[i] == '\\' || path[i] == '/')
      break;
  
  if (string == NULL && (string = malloc(l - i)) == NULL)
    return(NULL);

  strcpy(string,path + i + 1);
  return(string);
}

char *build_path(directory, filename)
     char *directory;
     char *filename;
{
  char *path;

  if (*filename == '/') {
    path = (char *) xmalloc(strlen(filename)+1);
    strcpy(path,filename);
  }
  else {
    path = (char *) xmalloc(strlen(directory)+1+strlen(filename)+1);
    sprintf(path,"%s/%s",directory,filename);
  }
  return(path);
}

FILE *xfopen(file, mode)
     char *file;
     char *mode;
{
  FILE *fp, *fopen();
  extern int errno, sys_nerr;
  extern char *program, *sys_errlist[];
  
  if ((fp = fopen(file,mode)) == NULL) {
    fprintf(stderr,"%s: can't open file '%s' mode '%s'",
	    program, file, mode);
    if (errno > 0 && errno < sys_nerr)
      fprintf(stderr," (%s)",sys_errlist[errno]);
    fprintf(stderr,"\n");
    exit(-1);
  }
  return(fp);
}

