/*
Copyright (C) 1995 M. A. S. Elmohamed,
Northeast Parallel Architectures Center, Syracuse University.
saleh@npac.syr.edu
*/

#ifndef DEFINED_PI_H
#define DEFINED_PI_H

/* **********************************************************************/
/* ANSI -- because we were too lazy to lint */
#ifndef _PARAMS
#if defined(ANSI) || defined(_ANSI_) || defined(__STDC__)
#define _PARAMS(args) args
#else
#define _PARAMS(args) ()
#endif
#endif /* _PARAMS */

#ifndef _K_AND_R_C_
#if !(defined(ANSI) || defined(_ANSI_) || defined(__STDC__))
#define _K_AND_R_C_
#define void
#endif
#endif /* _K_AND_R_C_ */
/* **********************************************************************/

#define PIA_INCREMENT	25	/* re-allocation increment */
#define FTA_INCREMENT	5	/* re-allocation increment */

#define SECS_PER_DAY	(24L * 60L * 60L) /* Seconds per day */

typedef long ep_t;		/* epoch data type holds time info */


/* The free time datum */
typedef struct ft_struc {
  ep_t bt;			/* begin time */
  ep_t et;			/* end time */
  int assigned;			/* time slot is assigned */
} ft_s, *ft_t;

/* The free time array */
typedef struct fta_struc {
  ft_t *ft;			/* free time data */
  int nft;			/* number of items in array */
  int nft_limit;		/* maximum number that will fit in array */
} fta_s, *fta_t;

/* The personal info datum */
typedef struct pi_struct {
  char *name;			/* person's name */
  fta_t fta;			/* array of free time */
  fta_t rfta;			/* array of restricted free time */
  fta_t sfta;			/* array of solution time */
} pi_s, *pi_t;

/* The personal info array */
typedef struct pia_struc {
  pi_t *pi;			/* person info data */
  int npi;			/* number of person data */
  int npi_limit;	
} pia_s, *pia_t;

ft_t ft_create _PARAMS((ep_t start, ep_t stop, int assigned));
ft_t ft_set _PARAMS((ft_t ft, ep_t start, ep_t stop, int assigned));
void ft_destroy _PARAMS((ft_t ft));
char *ft_time_string _PARAMS((int twelve_hour, ep_t bt, ep_t et, char *buf));

fta_t fta_create _PARAMS((void));
void fta_deleteft _PARAMS((fta_t fta, int ix));
void fta_removeft _PARAMS((fta_t fta, int ix));
void fta_sort _PARAMS((fta_t fta));
void fta_intersect _PARAMS((fta_t src_fta, fta_t dst_fta));
#define fta_getlength(fta) 		(fta->nft)
#define fta_setlength(fta,n)		(fta->nft = (n))

#define fta_getft_array(fta)		(fta->ft)
ft_t fta_add_time _PARAMS((fta_t fta, ep_t start, ep_t stop, int assigned));
int fta_add_stringtime _PARAMS((fta_t fta,char *time));

pi_t pi_create _PARAMS((char *name));
#define pi_getfta(pi)			(pi->fta)
#define pi_getrfta(pi)			(pi->rfta)
#define pi_getsfta(pi)			(pi->sfta)
#define pi_add_stringtime(pi,time)	fta_add_stringtime(pi->fta,time)
#define pi_getname(pi)			(pi->name)

pia_t pia_create _PARAMS((void));
#define pia_getlength(pia) 		(pia->npi)
#define pia_getpi_elt(pia,i)		(pia->pi[i])
pi_t pia_add_person _PARAMS((pia_t pia, char *name));
pi_t pia_find_person _PARAMS((pia_t pia, char *name));
void pia_sort _PARAMS((pia_t pia));
void pia_sort_on_rfta _PARAMS((pia_t pia));

int conv_string_interval24 _PARAMS((char *time, ep_t *start, ep_t *stop, int *assigned));
char *conv_interval24_string _PARAMS((int twelve_hour,ep_t bt, ep_t et, char *buf));

#endif /* DEFINED_PI_H */
