/*
Copyright (C) 1995 M. A. S. Elmohamed,
Northeast Parallel Architectures Center, Syracuse University.
saleh@npac.syr.edu
*/

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include "misc.h"
#include "config.h"
#include "applvar.h"
#include "pi.h"
#include "st.h"
#include "tg.h"
#include "arsched.h"
#include "print.h"

/* ********************************************************************** */

void print_student_schedules(FILE *fp, pia_t pia, fta_t slot_fta)
{
  char buf[SCRATCH_BUF_LEN];
  int nstudents = pia_getlength(pia);
  int slot_nft = fta_getlength(slot_fta), stud_nft;
  int i, slot_ix,stud_ix;
  ep_t stud_beg, stud_end, slot_beg, slot_end;
  pi_t pi;
  ft_t *slot_ft = fta_getft_array(slot_fta), *stud_ft;
  fta_t stud_fta;

  for (i = 0; i < nstudents; i++) {
    pi = pia_getpi_elt(pia,i);
    fprintf(fp,"%15.15s: ",pi_getname(pi));
    stud_fta = pi_getfta(pi);
    stud_ft = fta_getft_array(stud_fta);
    stud_nft = fta_getlength(stud_fta);

    for (stud_ix = 0, slot_ix = 0; stud_ix < stud_nft && slot_ix < slot_nft;) {
      stud_beg = stud_ft[stud_ix]->bt;
      stud_end = stud_ft[stud_ix]->et;
      slot_beg = slot_ft[slot_ix]->bt;
      slot_end = slot_ft[slot_ix]->et;
      /* is slot time within student time schedule? */
      if (stud_beg <= slot_beg && stud_end >= slot_end) {
	ft_time_string(1,max(slot_ft[slot_ix]->bt,stud_ft[stud_ix]->bt),
		       min(slot_ft[slot_ix]->et,stud_ft[stud_ix]->et),buf);
	fprintf(fp,"%8.8s ",buf);
      }
      else
	fprintf(fp,"%8.8s ","XXXXXXXX");
      
      if (slot_ix % 8 == 7)
	fprintf(fp,"\n%15.15s  ","");
      
      /* move forward */
      if (slot_beg == stud_beg && slot_end == stud_end) {
	slot_ix++;
	stud_ix++;
      }
      else if ((slot_beg >= stud_beg && slot_end <= stud_end) ||
	  (slot_beg <= stud_beg && slot_end <= stud_end))
	slot_ix++;
      else
	stud_ix++;
    }
    /* kludge to complete the printout */
    for (; slot_ix < slot_nft; slot_ix++) {
      fprintf(fp,"%8.8s ","XXXXXXXX");
      
      if (slot_ix % 8 == 7)
	fprintf(fp,"\n%15.15s  ","");
    }

    fprintf(fp,"\n");
  }
}

/* ********************************************************************** */

void print_restricted_schedules(FILE *fp, pia_t pia, fta_t slot_fta)
{
  char buf[SCRATCH_BUF_LEN];
  int nstudents = pia_getlength(pia);
  int slot_nft = fta_getlength(slot_fta), stud_nft;
  int i, slot_ix, stud_ix;
  ep_t stud_beg, stud_end, slot_beg, slot_end;
  pi_t pi;
  ft_t *slot_ft = fta_getft_array(slot_fta), *stud_ft;
  fta_t stud_fta;

  for (i = 0; i < nstudents; i++) {
    pi = pia_getpi_elt(pia,i);
    fprintf(fp,"%15.15s: ",pi_getname(pi));
    stud_fta = pi_getrfta(pi);	/* restricted fta */
    stud_ft = fta_getft_array(stud_fta);
    stud_nft = fta_getlength(stud_fta);
    stud_beg = stud_end = 0;
    for (stud_ix = 0, slot_ix = 0; stud_ix < stud_nft && slot_ix < slot_nft;) {
      stud_beg = stud_ft[stud_ix]->bt;
      stud_end = stud_ft[stud_ix]->et;
      slot_beg = slot_ft[slot_ix]->bt;
      slot_end = slot_ft[slot_ix]->et;
      /* is slot time within student time schedule? */
      if (stud_beg <= slot_beg && stud_end >= slot_end) {
	ft_time_string(1,max(slot_ft[slot_ix]->bt,stud_ft[stud_ix]->bt),
		       min(slot_ft[slot_ix]->et,stud_ft[stud_ix]->et),buf);
	fprintf(fp,"%8.8s ",buf);
      }
      else
	fprintf(fp,"%8.8s ","XXXXXXXX");
      
      if (slot_ix % 8 == 7)
	fprintf(fp,"\n%15.15s  ","");
      
      /* move forward */
      if (slot_beg == stud_beg && slot_end == stud_end) {
	slot_ix++;
	stud_ix++;
      }
      else if ((slot_beg >= stud_beg && slot_end <= stud_end) ||
	  (slot_beg <= stud_beg && slot_end <= stud_end))
	slot_ix++;
      else
	stud_ix++;
    }
    /* kludge to complete the printout */
    for (; slot_ix < slot_nft; slot_ix++) {
      fprintf(fp,"%8.8s ","XXXXXXXX");
      
      if (slot_ix % 8 == 7)
	fprintf(fp,"\n%15.15s  ","");
    }

    fprintf(fp,"\n");
  }
}

/* ********************************************************************** */

void print_solution_schedules(FILE *fp, pia_t pia, fta_t slot_fta)
{
  char buf[SCRATCH_BUF_LEN];
  int nstudents = pia_getlength(pia);
  int slot_nft = fta_getlength(slot_fta), stud_nft;
  int assigned, i, slot_ix, stud_ix;
  ep_t stud_beg, stud_end, slot_beg, slot_end;
  pi_t pi;
  ft_t *slot_ft = fta_getft_array(slot_fta), *stud_ft;
  fta_t stud_fta;

  for (i = 0; i < nstudents; i++) {
    pi = pia_getpi_elt(pia,i);
    fprintf(fp,"%15.15s: ",pi_getname(pi));
    stud_fta = pi_getsfta(pi);	/* solution fta */
    stud_ft = fta_getft_array(stud_fta);
    stud_nft = fta_getlength(stud_fta);
    stud_beg = stud_end = 0;
    for (stud_ix = 0, slot_ix = 0; stud_ix < stud_nft && slot_ix < slot_nft;) {
      stud_beg = stud_ft[stud_ix]->bt;
      stud_end = stud_ft[stud_ix]->et;
      slot_beg = slot_ft[slot_ix]->bt;
      slot_end = slot_ft[slot_ix]->et;
      assigned = stud_ft[stud_ix]->assigned;
      /* is slot time within student time schedule? */
      if (stud_beg <= slot_beg && stud_end >= slot_end && assigned) {
	ft_time_string(1,max(slot_ft[slot_ix]->bt,stud_ft[stud_ix]->bt),
		       min(slot_ft[slot_ix]->et,stud_ft[stud_ix]->et),buf);
	fprintf(fp,"%8.8s ",buf);
      }
      else
	fprintf(fp,"%8.8s ","XXXXXXXX");
      
      if (slot_ix % 8 == 7)
	fprintf(fp,"\n%15.15s  ","");
      
      /* move forward */
      if (slot_beg == stud_beg && slot_end == stud_end) {
	slot_ix++;
	stud_ix++;
      }
      else if ((slot_beg >= stud_beg && slot_end <= stud_end) ||
	  (slot_beg <= stud_beg && slot_end <= stud_end))
	slot_ix++;
      else
	stud_ix++;
    }
    /* kludge to complete the printout */
    for (; slot_ix < slot_nft; slot_ix++) {
      fprintf(fp,"%8.8s ","XXXXXXXX");
      
      if (slot_ix % 8 == 7)
	fprintf(fp,"\n%15.15s  ","");
    }

    fprintf(fp,"\n");
  }
}

/* ********************************************************************** */

void print_professor_schedules(FILE *fp, pia_t pia, fta_t slot_fta)
{
  char buf[SCRATCH_BUF_LEN];
  int nprofessors = pia_getlength(pia);
  int slot_nft = fta_getlength(slot_fta), prof_nft;
  int i, slot_ix, prof_ix;
  ep_t prof_beg, prof_end, slot_beg, slot_end;
  pi_t pi;
  ft_t *slot_ft = fta_getft_array(slot_fta), *prof_ft;
  fta_t prof_fta;

  for (i = 0; i < nprofessors; i++) {
    pi = pia_getpi_elt(pia,i);
    fprintf(fp,"%15.15s: ",pi_getname(pi));
    prof_fta = pi_getfta(pi);
    prof_ft = fta_getft_array(prof_fta);
    prof_nft = fta_getlength(prof_fta);
    for (prof_ix = 0, slot_ix = 0; prof_ix < prof_nft && slot_ix < slot_nft;) {
      prof_beg = prof_ft[prof_ix]->bt;
      prof_end = prof_ft[prof_ix]->et;
      slot_beg = slot_ft[slot_ix]->bt;
      slot_end = slot_ft[slot_ix]->et;
      /* is slot time within professor time schedule? */
      if (prof_beg <= slot_beg && prof_end >= slot_end) {
	ft_time_string(1,max(slot_ft[slot_ix]->bt,prof_ft[prof_ix]->bt),
		       min(slot_ft[slot_ix]->et,prof_ft[prof_ix]->et),buf);
	fprintf(fp,"%8.8s ",buf);
      }
      else
	fprintf(fp,"%8.8s ","XXXXXXXX");

      if (slot_ix % 8 == 7)
	fprintf(fp,"\n%15.15s  ","");
      
      /* move forward */
      if (slot_beg == prof_beg && slot_end == prof_end) {
	slot_ix++;
	prof_ix++;
      }
      else if ((slot_beg >= prof_beg && slot_end <= prof_end) ||
	  (slot_beg <= prof_beg && slot_end <= prof_end))
	slot_ix++;
      else
	prof_ix++;
    }
    /* kludge to complete the printout */
    for (; slot_ix < slot_nft; slot_ix++) {
      fprintf(fp,"%8.8s ","XXXXXXXX");
      
      if (slot_ix % 8 == 7)
	fprintf(fp,"\n%15.15s  ","");
    }

    fprintf(fp,"\n");
  }
}

/* ********************************************************************** */

void print_student_report(FILE *fp, pia_t pia)
{
  char buf[SCRATCH_BUF_LEN];
  int nstudents = pia_getlength(pia);
  int i, j, stud_nft;
  pi_t pi;
  ft_t *stud_ft;
  fta_t stud_fta;

  for (i = 0; i < nstudents; i++) {
    pi = pia_getpi_elt(pia,i);
    fprintf(fp,"%15.15s: ",pi_getname(pi));
    stud_fta = pi_getsfta(pi);	/* solution fta */
    stud_ft = fta_getft_array(stud_fta);
    stud_nft = fta_getlength(stud_fta);
    for (j = 0; j < stud_nft; j++)
      if (stud_ft[j]->assigned) {
	fprintf(fp,"%8.8s ",
		ft_time_string(1,stud_ft[j]->bt,stud_ft[j]->et,buf));
	break;			/* There is only one */
      }
    fprintf(fp,"\n");
  }
}
	
/* ********************************************************************** */

void print_professors_report(FILE *fp,  
                             pia_t stud_pia, 
                             pia_t prof_pia, 
                             tga_t tga)
{
  char buf[SCRATCH_BUF_LEN], *prof_name, *stud_name;
  int prof_npi = pia_getlength(prof_pia);
  int ntg = tga_getlength(tga);
  int i, j, k, stud_nft;
  pi_t stud_pi, prof_pi;
  ft_t *stud_ft;
  fta_t stud_fta;
  tg_t tg;
  sta_t prof_sta;
  int prof_nst;
  st_t *prof_st;

  /* 
     For each professor, we collect all the students that have
     her as an advisor in a scratch sta (tsta), along with the
     students name, so that we can time sort the information 
     before printing.  If sorting was not important, we wouldn't
     need to create (nor use) the new datatype (sta).
     */

  for (i = 0; i < prof_npi; i++) {
    prof_pi = pia_getpi_elt(prof_pia,i);
    prof_name = pi_getname(prof_pi);
    fprintf(fp,"------------------------\n");    
    fprintf(fp,"%15.15s\n",prof_name);
    fprintf(fp,"------------------------\n");    
    prof_sta = sta_create();
    for (j = 0; j < ntg; j++) {
      tg = tga_gettg_elt(tga,j);
      if (tg_find_professor(tg, prof_name) != (gma_t)0) {
	stud_name = tg_getname(tg);
	stud_pi = pia_find_person(stud_pia,stud_name);
	stud_fta = pi_getsfta(stud_pi);
	stud_ft = fta_getft_array(stud_fta);
	stud_nft = fta_getlength(stud_fta);
	for (k = 0; k < stud_nft; k++)
	  if (stud_ft[k]->assigned) {
	    sta_add(prof_sta, stud_name, stud_ft[k]);
	    break;			/* There is only one */
	  }
      }
    }
    sta_time_sort(prof_sta);
    prof_nst = sta_getlength(prof_sta);
    prof_st = sta_getst_array(prof_sta);
    for (j = 0; j < prof_nst; j++) {
      fprintf(fp,"%15.15s ",prof_st[j]->name);
      fprintf(fp,"%8.8s\n",
	      ft_time_string(1,prof_st[j]->ft->bt,prof_st[j]->ft->et,buf));
    }
    fprintf(fp,"\n");
    sta_destroy(prof_sta);
  }
}

/* ********************************************************************** */

void print_time_slots_report(FILE *fp, pia_t stud_pia, fta_t  slot_fta)
{
  char buf[SCRATCH_BUF_LEN], *stud_name;
  int stud_npi = pia_getlength(stud_pia);
  int i, j, k, stud_nft, slot_nft = fta_getlength(slot_fta);
  ft_t *stud_ft, *slot_ft = fta_getft_array(slot_fta);
  fta_t stud_fta;
  pi_t stud_pi;

  for (i = 0; i < slot_nft; i++) {
    fprintf(fp,"---------------\n");
    fprintf(fp,"%15.15s\n",ft_time_string(1,slot_ft[i]->bt,slot_ft[i]->et,buf));
    fprintf(fp,"---------------\n");
    for (j = 0; j < stud_npi; j++) {
      stud_pi = pia_getpi_elt(stud_pia,j);
      stud_name = pi_getname(stud_pi);
      stud_fta = pi_getsfta(stud_pi);	/* the solutions fta */
      stud_nft = fta_getlength(stud_fta);
      stud_ft = fta_getft_array(stud_fta);
      for (k = 0; k < stud_nft; k++) {
	if (stud_ft[k]->assigned && 
	    stud_ft[k]->bt == slot_ft[i]->bt && 
	    stud_ft[k]->et == slot_ft[i]->et) {
	  fprintf(fp,"%15.15s\n",stud_name);
	  break;
	}
      }
    }
    fprintf(fp,"\n");
  }
}
