/*
Copyright (C) 1995 M. A. S. Elmohamed,
Northeast Parallel Architectures Center, Syracuse University.
saleh@npac.syr.edu
*/

#include <stdio.h>
#ifndef MSDOS
#include <strings.h>
#endif
#include "misc.h"
#include "config.h"
#include "tg.h"

gm_t gm_create(char *name)
{
  gm_t gm;

  gm = (gm_t) xmalloc(strlen(name)+1);
  strcpy(gm, name);
  return(gm);
}  

gma_t gma_create(void)
{
  gma_t gma = (gma_t)xcalloc(1,sizeof(gma_s));
  return(gma);
}

gm_t gma_add_member(gma_t gma, char *member)
{
  gm_t gm = (gm_t)0;

  if (gma != (gma_t)0) {
    if (gma->ngm >= gma->ngm_limit) { /* need more array space */
      if (gma->gm == (gm_t *)0)
	gma->gm = (gm_t *)
	  xmalloc((gma->ngm_limit + GMA_INCREMENT) * sizeof(gm_t));
      else
	gma->gm = (gm_t *) 
	  xrealloc((char *) gma->gm,
		   (gma->ngm_limit + GMA_INCREMENT) * sizeof(gm_t));
      gma->ngm_limit += GMA_INCREMENT;
    }
    gma->gm[gma->ngm++] = gm = gm_create(member);
  }
  return(gm);
}

tg_t tg_create(char *name)
{
  tg_t tg = (tg_t) xcalloc(1,sizeof(tg_s));
  tg->gma = gma_create();
  gma_add_member(tg->gma, name);
  return(tg);
}

gma_t tg_add_professor(tg_t tg, char *name)
{
  if (tg != (tg_t)0) {
    gma_add_member(tg->gma, name);
    return(tg->gma);
  }
  return((gma_t)0);
}

gma_t tg_find_student(tg_t tg, char *name)
{
  gma_t gma;
  gma = tg->gma;
  if (gma->ngm > 0)
    if (strcasecmp(gma->gm[0],name)==0)
      return(gma);
  return((gma_t)0);
}


gma_t tg_find_professor(tg_t tg, char *name)
{
  gma_t gma;
  register int i;
  gma = tg->gma;
  for (i = 1; i < gma->ngm; i++)
    if (strcasecmp(gma->gm[i],name)==0)
      return(gma);
  return((gma_t)0);
}


tg_t tga_find_student(tga_t tga, char *name)
{
  register int i;
  tg_t tg;

  if (tga != (tga_t)0) {
    for (i = 0; i < tga->ntg; i++) {
      tg = tga->tg[i];
      if (tg_find_student(tg, name) != (gma_t)0)
	return(tg);
    }
  }
  return((tg_t)0);
}


tg_t tga_find_professor(tga_t tga, char *name)
{
  register int i;
  tg_t tg;

  if (tga != (tga_t)0) {
    for (i = 0; i < tga->ntg; i++) {
      tg = tga->tg[i];
      if (tg_find_professor(tg, name) != (gma_t)0)
	return(tg);
    }
  }
  return((tg_t)0);
}


tg_t tga_add_student(tga_t tga, char *name)
{
  tg_t tg = (tg_t)0;

  if (tga != (tga_t)0) {
    if (tga->ntg >= tga->ntg_limit) { /* need more array space */
      if (tga->tg == (tg_t *)0)
	tga->tg = (tg_t *)
	  xmalloc((tga->ntg_limit + TGA_INCREMENT) * sizeof(tg_t));
      else
	tga->tg = (tg_t *) 
	  xrealloc((char *)tga->tg, 
		   (tga->ntg_limit + TGA_INCREMENT) * sizeof(tg_t));
      tga->ntg_limit += TGA_INCREMENT;
    }
    tga->tg[tga->ntg++] = tg = tg_create(name);
  }
  return(tg);
}


void tga_sort(tga_t tga)
{
  int done, i, tgalen = tga->ntg;
  tg_t *tg = tga->tg, ttg;
  
  done = 0;
  while (done == 0) {
    done = 1;
    for (i = 0; i < tgalen - 1; i++) {
      if (strcasecmp(tg[i]->gma->gm[0],tg[i+1]->gma->gm[0])>0) {
	ttg = tg[i];
	tg[i] = tg[i+1];
	tg[i+1] = ttg;
	done = 0;
      }
    }
  }
}

tga_t tga_create(void)
{
  tga_t tga = (tga_t)xcalloc(1,sizeof(tga_s));
  return(tga);
}

